//+------------------------------------------------------------------+
//|                                                 StepChart_v1.mq4 |
//|                                Copyright  2010, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2010, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_width1 3
#property indicator_width2 3
//---- input parameters
extern int     Price        =     0; //Applied Price: 0-C,1-O,2-H,3-L,4-Median,5-Typical,6-Weighted
extern int     StepSize     =    2; //Size in pips 
extern color   ColorUp      = Blue;
extern color   ColorDown    = Red;
extern int     BarWidth     = 3;
extern string  UniqueID     = "StepChart";
extern bool    ShowPriceTag = true;
extern bool    alertsOn        = false;
extern bool    alertsOnCurrent = false;
extern bool    alertsMessage   = true;
extern bool    alertsSound     = false;
extern bool    alertsEmail     = false;

//---- indicator buffers
double UpBuffer[];
double DnBuffer[];
double sPrice[];
double trend[];
string short_name;
string name;
int    window;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   name = UniqueID+":";
//---- indicator line
   IndicatorBuffers(4);
   SetIndexBuffer(0,UpBuffer);
   SetIndexBuffer(1,DnBuffer);
   SetIndexBuffer(2,sPrice);
   SetIndexBuffer(3,trend);
   SetIndexStyle(0,DRAW_NONE);
   SetIndexStyle(1,DRAW_NONE);
   
//---- name for DataWindow and indicator subwindow label
   short_name=UniqueID+" StepChart("+Price+","+StepSize+")";
   IndicatorShortName(short_name);
//----
   SetIndexDrawBegin(0,2);
   SetIndexDrawBegin(1,2);
   
//----
   return(0);
}

int deinit()
{
   string lookFor       = name;
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}

//+------------------------------------------------------------------+
//| StepChart_v1                                                     |
//+------------------------------------------------------------------+
int start()
{
   
   int shift,limit, counted_bars=IndicatorCounted();
   double _Point = Point * MathPow(10,Digits%2);
   
   if ( counted_bars > 0 )  limit=Bars-counted_bars-1;
   if ( counted_bars < 0 )  return(0);
   if ( counted_bars ==0 )  limit=Bars-1; 
     
	
	if (counted_bars < 1)
   { 
      for(int i=0;i<Bars-1;i++)
      { 
      UpBuffer[i]=EMPTY_VALUE;
      DnBuffer[i]=EMPTY_VALUE;
      }
   }   
	

   window = WindowFind(short_name);   
	for(shift=limit;shift>=0;shift--) 
   {	
   double Step = StepSize * _Point;
   double price = iMA(NULL,0,1,0,0,Price,shift);
   
      if(shift < Bars-1)
      {
      sPrice[shift] = sPrice[shift+1];   
         if(MathAbs(price - sPrice[shift]) < Step) sPrice[shift] = sPrice[shift+1]; 
         else
         { 
            while(MathAbs(price - sPrice[shift]) > Step) 
            {
    	      if(price > sPrice[shift]) sPrice[shift] = sPrice[shift] + Step;
		      if(price < sPrice[shift]) sPrice[shift] = sPrice[shift] - Step;
		      }
		   }    
	     
         trend[shift] = trend[shift+1];
         if(sPrice[shift] > sPrice[shift+1]) trend[shift] = 1;
         if(sPrice[shift] < sPrice[shift+1]) trend[shift] =-1;   
         
         if(trend[shift] > 0)   
         {
         UpBuffer[shift] = sPrice[shift];
            if(sPrice[shift] == sPrice[shift+1]) DnBuffer[shift] = sPrice[shift] - Point;  
            else DnBuffer[shift] = sPrice[shift+1];
         }
         else
         if(trend[shift] < 0)   
         {
         UpBuffer[shift] = sPrice[shift]; 
            if(sPrice[shift] == sPrice[shift+1]) DnBuffer[shift] = sPrice[shift+1] + Point;  
            else DnBuffer[shift] = sPrice[shift+1];
         }

         //
         //
         //
         //
         //
         
         color theColor = ColorUp;
            if (DnBuffer[shift]>UpBuffer[shift]) theColor = ColorDown;
            drawBar(Time[shift],MathMax(UpBuffer[shift],DnBuffer[shift]),MathMin(UpBuffer[shift],DnBuffer[shift]),theColor,BarWidth);
      }
      else
      { 
      sPrice[shift] = MathRound(price/(StepSize * _Point)) * StepSize * _Point;         
      trend[shift] = 0;
      }
   }      

   //
   //
   //
   //
   //
   
   if (ShowPriceTag)
   {
      if (trend[0] ==  1) drawPriceTag(MathMax(DnBuffer[0],UpBuffer[0]),ColorUp);
      if (trend[0] == -1) drawPriceTag(MathMin(DnBuffer[0],UpBuffer[0]),ColorDown);
   }
   manageAlerts();
   return(0);	
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void drawBar(int bTime, double prHigh, double prLow, color barColor,int barWidth)
{
   string oName;
          oName = name+TimeToStr(bTime)+":";
            ObjectCreate(oName,OBJ_TREND,window,bTime,0,bTime,0);
                 ObjectSet(oName, OBJPROP_PRICE1, prHigh);
                 ObjectSet(oName, OBJPROP_PRICE2, prLow);
                 ObjectSet(oName, OBJPROP_COLOR, barColor);
                 ObjectSet(oName, OBJPROP_WIDTH, barWidth);
                 ObjectSet(oName, OBJPROP_RAY, false);
                 ObjectSet(oName, OBJPROP_BACK, true);
}
void drawPriceTag(double price, color theColor)
{
   string oName;
          oName = name+":pt";
            ObjectCreate(oName,OBJ_ARROW,window,Time[0],0);
                 ObjectSet(oName, OBJPROP_PRICE1, price);
                 ObjectSet(oName, OBJPROP_TIME1, Time[0]);
                 ObjectSet(oName, OBJPROP_ARROWCODE,SYMBOL_RIGHTPRICE);
                 ObjectSet(oName, OBJPROP_COLOR, theColor);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
            if (trend[whichBar] ==  1) doAlert(whichBar,"up");
            if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," step chart - trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"step chart"),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}