//+------------------------------------------------------------------+
//|                                           T101 Basket Trader.mq4 |
//|                                  Copyright  2011, John Wustrack |
//|                                        john_wustrack@hotmail.com |
//|                                                                  |
//| v1.0 - 06.09.11 - Base version                                   |
//| v1.1 - 07.09.11 - Added trailing                                 |
//|                   Fixed problem with rank values                 |
//|                   Changed min profit to breakeven                |
//|                   Added confirmation msgbox's to important       |
//|                   buttons                                        |
//|                   Added ability for pending buy/sell             |
//|                   Fixed multiple alert bug                       |
//| v1.2 - 07.09.11 - Show missing/hedged basket pairs               |
//|                   If no open basket, show general basket         |
//|                   balance                                        |
//|                   Improved robustness of opening & closing of    |
//|                   orders                                         |
//|                   Fixed currency list for basket 14              |
//|                   Remove visual nbrs for tp,sl,hedge             |
//|                   Show "PROCESSING" during open/close            |
//|                   Add ability for balance threshold protection   |
//| V1.3 - 08.09.11 - Ability to exclude pairs from basket           |
//|                   Ability to trade certain pairs opposite        |
//|                   Fixed problem with "extra" data being          |
//|                   displayed when using less than 14 pairs        |
//|                   Improved break even & lock handling            |
//|                   Moved start trailing to GVariable              |
//|                                                                  |
//|                                                                  |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, John Wustrack"
#property link      "john_wustrack@hotmail.com"


//+------------------------------------------------------------------+
//| G E N E R A L   D E F I N I T I O N S                            |
//+------------------------------------------------------------------+
#include <WinUser32.mqh>

#define Version "v1.3 - 08 Sept 2011"
#define EAComment "T101 Basket Trader"
#define ObjPfx "T101 Bskt Trdr"
//#define Breakeven_GVar "!T101 Breakeven Value"
#define DftTrail_GVar "!T101 Dft Start Trail at"
#define DftBrkEv_GVar "!T101 Dft B/E+Lock at"
#define Trailing_GVar "!T101 Trailing Value"
#define StopLoss_GVar "!T101 Stop Loss Value"
#define TakeProfit_GVar "!T101 Take Profit Value"
#define Hedge_GVar "!T101 Hedge Value"
#define Exclusion_GVar "!T101 Exclude "
#define ArrowPfx "T101 Trade Arrow"
#define DirectionPfx "NB 10.0 Basis - "
#define SlippagePts 30
#define DefaultDirection 0
#define StdCurrencyPair "EURUSD"
#define OfflineChart "##T101##"
#define DftPendingDistance 150
#define MaxPairs 14
#define MaxRetries 60
#define NL "\n"

// Screen Definitions 
#define DirButX 10
#define DirButY 20
#define BuyButX 10
#define BuyButY 50
#define BuyText "Buy Basket"
#define SellButX 10
#define SellButY 80
#define SellText "Sell Basket"
#define CloseButX 10
#define CloseButY 110
#define CloseText "Close Basket"
#define BreakEvButX 10
#define BreakEvButY 140
#define BreakEvText "Set to Breakeven"
#define PenBuyButX 10
#define PenBuyButY 40
#define PenBuyText "Pending Buy"
#define PenSellButX 10
#define PenSellButY 10
#define PenSellText "Pending Sell"


#define PandLX 10
#define PandLY 170

#define RankStartY 200
#define MisPairStartY 320
#define RankStartOffY 25
#define RankFontSize 8
int RankX[MaxPairs] = {125,10,125,10,125,10,125,10,125,10,125,10,125,10};
int RankOffY[MaxPairs] = {0,15,0,15,0,15,0,15,0,15,0,15,0,15};
#define TotBalanceX 10
#define TotBalanceY 320


#define ButtonFont "Arial Bold"
#define ButtonSize 18
#define DftFont "Arial"
#define DftFontSize 14
#define DftTextColor LightGray
#define BuyColor Blue
#define SellColor Red
#define CloseArrowColor Goldenrod
#define HedgeColor Orange
#define SLColor Red
#define TPColor Lime
#define LossColor Red
#define ProfitColor Lime
#define DirectionColor Yellow
#define HedgeText "Hedge"
#define SLText "Stoploss"
#define TPText "TakeProfit"

#define ProcessingFont "Arial Bold"
#define ProcessingSize 64
#define ProcessingColor Yellow
#define ProcessingX 240
#define ProcessingY 70


//+------------------------------------------------------------------+
//| E N T R Y   P A R A M E T E R S                                  |
//+------------------------------------------------------------------+
extern string     xs_0001 = "--T101 Parameters";
extern int        PairsInBasket = 14;
extern string     PairSuffix = "";
extern string     PairsToTradeOpposite = "USDCHF,USDJPY";

extern string     xs_0002 = "--Trade/Trend Detection";
extern bool       UseNanningBobBasis = false;
extern bool       UseTradeConditions = false;

extern string     xs_0003 = "--Basket Parameters";
extern double     LotSize = 0.01;
extern int        BasketMagicNbr = 1010101;

extern string     xs_0004 = "--Take Profit";
extern bool       UseTakeProfit = true;
extern bool       VisualTP = true;
extern int        DefaultTP = 500;
extern bool       ValueTP = true;
extern double     DefaultTPValue = 100;

extern string     xs_0005 = "--Stop Loss";
extern bool       UseStopLoss = false;
extern bool       VisualSL = true;
extern int        DefaultSL = 500;
extern bool       ValueSL = true;
extern double     DefaultSLValue = 100;

extern string     xs_0006 = "--Trailing & BE+Lock";
extern bool       UseTrailing = true;
//extern double     MinTrailingValue = 15;
extern bool       TrailFixed = false;
extern double     FixedTrailValue = 10;
extern bool       TrailPercent = true;
extern double     TrailPrcntage = 75;
extern string     xs_0006a = "--Trailing/BE+Lock Defaults";
extern double     DftTrailingAtValue = 25;
extern double     DftBreakevenAtValue = 10;
extern double     DftLockValue = 5;

extern string     xs_0007 = "--Hedging";
extern bool       AllowHedging = true;
extern double     PairMinLoss4Hedge = 10;
extern bool       VisualHedge = true;
extern int        DefaultHedge = 500;
extern bool       ValueHedge = true;
extern double     DefaultHedgeValue = 100;

/*
extern string     xs_0008 = "--Basket Threshold Protection";
extern bool       UseBasketThreshold = true;
extern int        BasketThreshold = 700;
extern int        PairThreshold = 70;
*/

extern string     xs_0009 = "--Basket Balancing";
extern int        BasketBalanceMA = 3;
extern int        BasketBalanceLB = 5;

extern string     xs_0010 = "--Other Parameters";
extern int        SleepSecs = 1;



//+------------------------------------------------------------------+
//| G L O B A L   V A R I A B L E S                                  |
//+------------------------------------------------------------------+
string TradePair[MaxPairs];
string PairExcluded[MaxPairs];
int TradePairOpp[MaxPairs];
string OppTradePairs[MaxPairs];
double PairRank[MaxPairs][2];
double PairRank2[MaxPairs][2];
double PairRank3[MaxPairs][2];
double PairStrength[MaxPairs];

string DirText[7] = {"None","Long","Short","Range","CT Long","CT Short","n/a"};
#define MaxDirection 5

bool gb_Stopped;
bool gb_OpenBasket;
bool gb_BuyAllowed;
bool gb_SellAllowed;
bool gb_PendingExists;
bool gb_ExcludedPairs;

double gd_Profit.Loss;
double gd_BasketBalance;
double gd_BasketBalPc;
double gi_Type;

int Direction;

datetime TodaysDate;
datetime LastAlert;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   // Temporary
   AllowHedging = false;     
//   UseBasketThreshold = false;

   if (PairsInBasket != 1 &&
       PairsInBasket != 2 &&
       PairsInBasket != 4 &&
       PairsInBasket != 6 &&
       PairsInBasket != 8 &&
       PairsInBasket != 10 &&
       PairsInBasket != 12 &&
       PairsInBasket != 14)
      {
      Alert("Invalid number of pairs in the basket, EA is stopping!");
      gb_Stopped = true;
      return(0);
      }
      
   // If Hedging is on then SL must be off
   if (AllowHedging && UseStopLoss)
      {
      Alert("Hedging & SL cannot be used together, EA is stopping!");
      gb_Stopped = true;
      return(0);
      }
  
   // If this is not the T101 offline chart then stop
   if (Symbol() != OfflineChart)  
      {
      Alert("This EA can ONLY be run on the T101 offline chart, EA is stopping!");
      gb_Stopped = true;
      return(0);
      }
      
   // If the user has selected fixed trailing and percentage - use fixed
   if (UseTrailing)
      {
      if (TrailFixed && TrailPercent) TrailPercent = false;
      if (!TrailFixed && !TrailPercent) TrailFixed = true;
      }
  
   Get_Trade_Pairs();
   
   TodaysDate = iTime(StdCurrencyPair+PairSuffix,PERIOD_D1,0);
   
   Direction = DefaultDirection;
   
   if (!UseNanningBobBasis) Direction = 6;
   
   // Delete any Pending lines that may exist
   ObjectDelete(ObjPfx+PenBuyText+"_VL");
   ObjectDelete(ObjPfx+PenSellText+"_VL");
   
   
   // Create / Display the buttons
   Object_Create(ObjPfx+"Dir",DirButX,DirButY,DirectionPfx+DirText[Direction],ButtonSize,ButtonFont,DirectionColor);
   Setup_Buy_Sell_Buttons();
   
   // Reset Exclusion variables
   Reset_Pair_Exclusion();
   
   // Set up the global variables for breakeven & trail defaults
   GlobalVariableSet(DftTrail_GVar,DftTrailingAtValue);
   GlobalVariableSet(DftBrkEv_GVar,DftBreakevenAtValue);
   
   // Force the start routine to run
   start();   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   Delete_Objects();
   Comment("");
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   bool lb_ans;

   while(true)
      {
      // Exit if the EA is stopped
      if (gb_Stopped) return(0); 
      RefreshRates();
      
      Recreate_Global_Variables();
      // Get the pair exclusions
      Get_Pair_Exclusions();
            
      // Display the comment information
      Display_Comments();
         
      gb_OpenBasket = Get_Basket_Trades();
      
      // Check if any pending exists
      gb_PendingExists = false;
      if (ObjectFind(ObjPfx+PenBuyText+"_VL") != (-1) ||
          ObjectFind(ObjPfx+PenSellText+"_VL") != (-1))
         gb_PendingExists = true;
            
      // If the day has changed force the user to set a direction again
      datetime ld_date = iTime(StdCurrencyPair+PairSuffix,PERIOD_D1,0);
      if (ld_date != TodaysDate)
         {
         Direction=0;
         if (!UseNanningBobBasis) Direction = 6;
         Setup_Buy_Sell_Buttons();
         Object_Create(ObjPfx+"Dir",DirButX,DirButY,DirectionPfx+DirText[Direction],ButtonSize,ButtonFont,DirectionColor);
         TodaysDate = ld_date;
         }
      
      // Delete the close basket button if no open trades
      if (!gb_OpenBasket)
         ObjectDelete(ObjPfx+"Close");
      
      // Check for change of the direction button
      if (ObjectGet(ObjPfx+"Dir",OBJPROP_XDISTANCE) != DirButX ||
          ObjectGet(ObjPfx+"Dir",OBJPROP_YDISTANCE) != DirButY)
         {
         Direction++;
         if (Direction > MaxDirection) Direction=0;
         if (!UseNanningBobBasis) Direction = 6;
         // Reset the other buttons and immediately loop
         Setup_Buy_Sell_Buttons();
         Object_Create(ObjPfx+"Dir",DirButX,DirButY,DirectionPfx+DirText[Direction],ButtonSize,ButtonFont,DirectionColor);
         continue;
         }
         
      // If we have an open basket, perform any automatic tasks first, then manual ones
      if (gb_OpenBasket)   
         {
         // Check for hedge
         if (AllowHedging) Check_4_Hedge();
         // Check for SL
         if (UseStopLoss) Check_4_SL();
         // Check for TP
         if (UseTakeProfit) Check_4_TP();
         // Check for close
         double ld_close = 0;
//         if (!UseTrailing)
//            ld_close = GlobalVariableGet(Breakeven_GVar);
//         else
         ld_close = GlobalVariableGet(Trailing_GVar);
         if (ld_close != 0)
            Check_4_BE(ld_close);
         // If we still have an open basket check for manual close 
         if (gb_OpenBasket &&
             (ObjectGet(ObjPfx+"Close",OBJPROP_XDISTANCE) != CloseButX ||
              ObjectGet(ObjPfx+"Close",OBJPROP_YDISTANCE) != CloseButY))   
            Close_Basket_PP();
         // If we still have an open basket check for setting breakeven
         if (gb_OpenBasket && 
             (ObjectGet(ObjPfx+"Breakeven",OBJPROP_XDISTANCE) != BreakEvButX ||
              ObjectGet(ObjPfx+"Breakeven",OBJPROP_YDISTANCE) != BreakEvButY))
            Set_Breakeven();   
            
         // Update the trailing amounts
         if (UseTrailing) Trail_Basket();
         
         // If we don't have an open basket anymore, reset the other buttons and immediately loop
         if (!gb_OpenBasket)
            {
            Setup_Buy_Sell_Buttons();
            }
         
         continue;
         }

      // Check for pre-existing pending orders 
      if (gb_PendingExists)
         {
         lb_ans = Open_Pendings();
         if (lb_ans) continue;
         }
         

      // Check for manual buy / sell 
      if (gb_BuyAllowed && ObjectFind(ObjPfx+"Buy") != (-1) &&
          (ObjectGet(ObjPfx+"Buy",OBJPROP_XDISTANCE) != BuyButX ||
           ObjectGet(ObjPfx+"Buy",OBJPROP_YDISTANCE) != BuyButY))   
         {
         Buy_Basket_PP();
         continue;
         }
         
      if (gb_SellAllowed && ObjectFind(ObjPfx+"Sell") != (-1) &&
          (ObjectGet(ObjPfx+"Sell",OBJPROP_XDISTANCE) != SellButX ||
           ObjectGet(ObjPfx+"Sell",OBJPROP_YDISTANCE) != SellButY))   
         {
         Sell_Basket_PP();
         continue;
         }
      
      // Check for manual pending buy / sell 
      if (gb_BuyAllowed && ObjectFind(ObjPfx+PenBuyText) != (-1) &&
          (ObjectGet(ObjPfx+PenBuyText,OBJPROP_XDISTANCE) != PenBuyButX ||
           ObjectGet(ObjPfx+PenBuyText,OBJPROP_YDISTANCE) != PenBuyButY))   
         {
         Pending_Buy_Basket_PP();
         continue;
         }
         
      if (gb_SellAllowed && ObjectFind(ObjPfx+PenSellText) != (-1) &&
          (ObjectGet(ObjPfx+PenSellText,OBJPROP_XDISTANCE) != PenSellButX ||
           ObjectGet(ObjPfx+PenSellText,OBJPROP_YDISTANCE) != PenSellButY))   
         {
         Pending_Sell_Basket_PP();
         continue;
         }

      // If we are here, then check if there are any new trade conditions
      if (UseTradeConditions)
         Check_Trade_Conditions();
         
      // Recreate pending buttons if necessary
      if (!gb_PendingExists)
         Create_Pending_Buttons();
      
      // Now we can sleep
      Sleep(SleepSecs * 1000);
      WindowRedraw();
      }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| get all open orders for the basket                               |
//+------------------------------------------------------------------+
bool Get_Basket_Trades()
  {
//----
   bool pb_Basket;
   color lc_Color;
   
   gd_Profit.Loss = 0;
   gi_Type = -1;
   int li_Pair;
   ArrayInitialize(PairRank,(-1));
   for (int i2=0; i2<PairsInBasket; i2++)
      PairRank[i2][0] = 0;
 
   // Get open orders   
   for (int i=0; i<=OrdersTotal(); i++)
      {
      if (OrderSelect(i-1,SELECT_BY_POS)==true)
         {
         // Not this magic number - ignore
         if (OrderMagicNumber() != BasketMagicNbr) continue;
         // Add to the Profit/Loss
         gd_Profit.Loss += OrderProfit() + OrderSwap() - OrderCommission();
         // Add to the pair rank
         li_Pair = Get_Pair_Index(OrderSymbol());
         if (li_Pair >= 0)
            {
            PairRank[li_Pair,0] += OrderProfit() + OrderSwap() - OrderCommission();
            PairRank[li_Pair,1] = li_Pair;
            }
               
         
         pb_Basket = true;
         // Save the order type, ensure we deal with opposites correctly
         if (Get_Opposite_Pair_Index(Symbol()) == (-1))     
            gi_Type = OrderType();
         else 
            {
            if (OrderType() == OP_BUY) gi_Type = OP_SELL;
            if (OrderType() == OP_SELL) gi_Type = OP_BUY;
            }
         }
      }
      
   // Show the Profit / Loss
   if (pb_Basket)
      {
      lc_Color = DftTextColor;
      if (gd_Profit.Loss < 0) lc_Color = LossColor;
      if (gd_Profit.Loss > 0) lc_Color = ProfitColor;
   
      string ls_Type;
      if (gi_Type == OP_BUY) ls_Type = "Buy Basket P/L ";
      if (gi_Type == OP_SELL) ls_Type = "Sell Basket P/L ";
     
      Object_Create(ObjPfx+"PandL",PandLX,PandLY,ls_Type+DoubleToStr(gd_Profit.Loss,2)+" "+AccountCurrency(),DftFontSize,DftFont,lc_Color);
      }
   else
      ObjectDelete(ObjPfx+"PandL");
   
   double ld_hedge,ld_sl,ld_tp;
   // Delete any existing Buy/Sell buttons and create the close button if it doesnt exist
   if (pb_Basket)
      {
      if (gi_Type == OP_BUY)
         {
         ld_hedge = Close[0] - DefaultHedge;
         ld_sl = Close[0] - DefaultSL;
         ld_tp = Close[0] + DefaultTP;
         }
      if (gi_Type == OP_SELL)
         {
         ld_hedge = Close[0] + DefaultHedge;
         ld_sl = Close[0] + DefaultSL;
         ld_tp = Close[0] - DefaultTP;
         }
         
      ObjectDelete(ObjPfx+"Buy");
      ObjectDelete(ObjPfx+"Sell");         
      ObjectDelete(ObjPfx+PenBuyText);
      ObjectDelete(ObjPfx+PenSellText);
      gb_BuyAllowed = false;
      gb_SellAllowed = false;
      if (ObjectFind(ObjPfx+"Close") == (-1))
         Object_Create(ObjPfx+"Close",CloseButX,CloseButY,CloseText,ButtonSize,ButtonFont,DftTextColor);
         
      // Create the breakeven button if it doesn't exist
      if (ObjectFind(ObjPfx+"Breakeven") == (-1))
         Object_Create(ObjPfx+"Breakeven",BreakEvButX,BreakEvButY,BreakEvText,ButtonSize,ButtonFont,DftTextColor);
      
   
      // Ensure all lines that are required are still there
      if (AllowHedging)
         if (ObjectFind(ObjPfx+"HedgeLine") == (-1) && VisualHedge)
            {
            ObjectCreate(ObjPfx+"HedgeLine",OBJ_HLINE,0,0,0);
            ObjectSet(ObjPfx+"HedgeLine",OBJPROP_PRICE1,ld_hedge);
            ObjectSet(ObjPfx+"HedgeLine",OBJPROP_COLOR,HedgeColor);
            ObjectSet(ObjPfx+"HedgeLine",OBJPROP_STYLE,STYLE_DASHDOT);
            }
      if (UseStopLoss)
         if (ObjectFind(ObjPfx+"StopLoss") == (-1) && VisualSL)
            {
            ObjectCreate(ObjPfx+"StopLoss",OBJ_HLINE,0,0,0);
            ObjectSet(ObjPfx+"StopLoss",OBJPROP_PRICE1,ld_sl);
            ObjectSet(ObjPfx+"StopLoss",OBJPROP_COLOR,SLColor);
            ObjectSet(ObjPfx+"StopLoss",OBJPROP_STYLE,STYLE_DASHDOT);
            }
      if (UseTakeProfit)
         if (ObjectFind(ObjPfx+"TakeProfit") == (-1) && VisualTP)
            {
            ObjectCreate(ObjPfx+"TakeProfit",OBJ_HLINE,0,0,0);
            ObjectSet(ObjPfx+"TakeProfit",OBJPROP_PRICE1,ld_tp);
            ObjectSet(ObjPfx+"TakeProfit",OBJPROP_COLOR,TPColor);
            ObjectSet(ObjPfx+"TakeProfit",OBJPROP_STYLE,STYLE_DASHDOT);
            }
      }
      
   // If we dont have an open basket
   if (!pb_Basket)
      {
      // Delete any hedge,sl,tp lines
      ObjectDelete(ObjPfx+"HedgeLine");
      ObjectDelete(ObjPfx+"StopLoss");
      ObjectDelete(ObjPfx+"TakeProfit");
      ObjectDelete(ObjPfx+"Close");
      ObjectDelete(ObjPfx+"Breakeven");         
      }
   
   
      
   // Show the Rankings
   Show_Pair_Rankings(pb_Basket);

//----
   return(pb_Basket);
  }
//+------------------------------------------------------------------+
//| setup the buy sell buttons based on the "direction"              |
//+------------------------------------------------------------------+
int Setup_Buy_Sell_Buttons()
  {
//----
   gb_BuyAllowed = false;
   gb_SellAllowed = false;

   // If the direction is none, then delete the buy & sell
   if (Direction == 0)
      {
      ObjectDelete(ObjPfx+"Buy");
      ObjectDelete(ObjPfx+"Sell");
      ObjectDelete(ObjPfx+PenBuyText);
      ObjectDelete(ObjPfx+PenSellText);
      return(0);
      }
      
   // If the direction is 1,3,4 or 5 we need a buy button
   if (Direction == 1 || Direction == 3 ||
       Direction == 4 || Direction == 5 || Direction == 6)
      {
      Object_Create(ObjPfx+"Buy",BuyButX,BuyButY,"Buy Basket",ButtonSize,ButtonFont,BuyColor);
      gb_BuyAllowed = true;
      }
       
   // If the direction is 2,3,4 or 5 we need a sell button
   if (Direction == 2 || Direction == 3 ||
       Direction == 4 || Direction == 5 || Direction == 6)
      {
      Object_Create(ObjPfx+"Sell",SellButX,SellButY,"Sell Basket",ButtonSize,ButtonFont,SellColor);
      gb_SellAllowed = true;
      }
      
   // If the direction is 1 we don't need a sell button
   if (Direction == 1)
      {
      ObjectDelete(ObjPfx+"Sell");
      ObjectDelete(ObjPfx+PenSellText);
      gb_SellAllowed = false;
      }
   // If the direction is 2 we don't need a buy button
   if (Direction == 2)
      {
      ObjectDelete(ObjPfx+"Buy");
      ObjectDelete(ObjPfx+PenBuyText);
      gb_BuyAllowed = false;
      }
      
   // Set up the pending buttons if they are allowed
   Create_Pending_Buttons();
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| setup the buy sell buttons based on the "direction"              |
//+------------------------------------------------------------------+
int Create_Pending_Buttons()
  {
//----
   if (!gb_PendingExists)
      {
      if (gb_BuyAllowed)
         Object_Create_2(ObjPfx+PenBuyText,PenBuyButX,PenBuyButY,PenBuyText,ButtonSize,ButtonFont,BuyColor);
      if (gb_SellAllowed)
         Object_Create_2(ObjPfx+PenSellText,PenSellButX,PenSellButY,PenSellText,ButtonSize,ButtonFont,SellColor);
      }
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| check if we need to open a hedge                                 |
//+------------------------------------------------------------------+
int Check_4_Hedge()
  {
//----
   // Visually based hedging
   if (VisualHedge)
      {
      double ld_hedge = ObjectGet(ObjPfx+"HedgeLine",OBJPROP_PRICE1);
      if (ld_hedge == 0) return(0);      
   
      if (gi_Type == OP_BUY) // Currently a buy open so we need to hedge with a sell
         if (Close[0] < ld_hedge)
            Sell_Basket();
         
      if (gi_Type == OP_SELL) // Currently a sell open so we need to hedge with a buy
         if (Close[0] > ld_hedge)
            Buy_Basket();
      }

   // Value based hedging
   if (ValueHedge)
      {   
      if (gi_Type == OP_BUY) // Currently a buy open so we need to hedge with a sell
         if (gd_Profit.Loss > DefaultHedgeValue)
            Sell_Basket();
         
      if (gi_Type == OP_SELL) // Currently a sell open so we need to hedge with a buy
         if (gd_Profit.Loss > DefaultHedgeValue)
            Buy_Basket();
      }
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| check if we need to close for sl                                 |
//+------------------------------------------------------------------+
int Check_4_SL()
  {
//----
   // Visual SL
   if (VisualSL)
      {
      double ld_sl = ObjectGet(ObjPfx+"StopLoss",OBJPROP_PRICE1);
      if (ld_sl == 0) return(0);
   
      if (gi_Type == OP_BUY) // Currently a buy open
         if (Close[0] < ld_sl)
            Close_Basket();
         
      if (gi_Type == OP_SELL) // Currently a sell open
         if (Close[0] > ld_sl)
            Close_Basket();
      }
   
   // Value SL
   if (ValueSL)
      {   
      if (gd_Profit.Loss > DefaultSLValue)
         Close_Basket();
      }
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| check if we need to close for tp                                 |
//+------------------------------------------------------------------+
int Check_4_TP()
  {
//----
   // Visual TP
   if (VisualTP)
      {
      double ld_tp = ObjectGet(ObjPfx+"TakeProfit",OBJPROP_PRICE1);
      if (ld_tp == 0) return(0);
   
      if (gi_Type == OP_BUY) // Currently a buy open
         if (Close[0] > ld_tp)
            Close_Basket();
         
      if (gi_Type == OP_SELL) // Currently a sell open
         if (Close[0] < ld_tp)
            Close_Basket();
      }

   // Value TP
   if (ValueTP)
      {
      if (gd_Profit.Loss > DefaultTPValue)
         Close_Basket();
      }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| check if we need to close for mininum profit                     |
//+------------------------------------------------------------------+
int Check_4_BE(double pd_MinP)
  {
//----
   // Value MP
   if (gd_Profit.Loss < pd_MinP && pd_MinP != 0)
      Close_Basket();   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| trail a basket                                                   |
//+------------------------------------------------------------------+
int Trail_Basket()
  {
//----
   // We only trail if passed the minimum profit
//   if (gd_Profit.Loss < MinTrailingValue) return(0);
   
   double ld_Trail = GlobalVariableGet(Trailing_GVar);
   double ld_BE.At = GlobalVariableGet(DftBrkEv_GVar);
   double ld_Trail.At = GlobalVariableGet(DftTrail_GVar);
   
   // Check if we need to set the breakeven + lock
   if (ld_Trail == 0)
      {
      if (gd_Profit.Loss > ld_BE.At)
         {
         ld_Trail = DftLockValue;
         GlobalVariableSet(Trailing_GVar,ld_Trail);
         }
      }
   
   // We only need to trail if passed the start trailing at
   if (gd_Profit.Loss < ld_Trail.At) return(0);
         
   double ld_NewTrail = 0;
   
   // Calculate the new trail value
   if (TrailFixed)
      ld_NewTrail = NormalizeDouble(gd_Profit.Loss - FixedTrailValue,2);
      
   if (TrailPercent)
      ld_NewTrail = NormalizeDouble(gd_Profit.Loss * (TrailPrcntage / 100),2);
   
   // If the new trailing value is higher than the current then update
   if (ld_NewTrail > ld_Trail)
      GlobalVariableSet(Trailing_GVar,ld_NewTrail);   
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| set the breakeven                                                |
//+------------------------------------------------------------------+
int Set_Breakeven()
  {
//----
   if (gd_Profit.Loss > DftLockValue)
      GlobalVariableSet(Trailing_GVar,DftLockValue);
   else
      Alert("Breakeven cannot be set as the current P/L is less than the minimum!");
   Object_Create(ObjPfx+"Breakeven",BreakEvButX,BreakEvButY,BreakEvText,ButtonSize,ButtonFont,DftTextColor);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| close a basket (pre-processing)                                  |
//+------------------------------------------------------------------+
int Close_Basket_PP()
  {
//----
   int li_Ans;
   
   li_Ans = MessageBox("Are you sure you wish to close the basket?","Basket Close Confirmation",MB_YESNO|MB_ICONQUESTION);
   if (li_Ans==IDYES)
      Close_Basket();
   else
      // Reset the button to original 
      Object_Create(ObjPfx+"Close",CloseButX,CloseButY,CloseText,ButtonSize,ButtonFont,DftTextColor);
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| buy a basket (pre-processing)                                    |
//+------------------------------------------------------------------+
int Buy_Basket_PP()
  {
//----
   int li_Ans;
   string ls_Msg;
   
   ls_Msg = "Are you sure you wish to open a BUY basket?";
   
   if (gb_ExcludedPairs)
      ls_Msg = ls_Msg + NL + NL + "Warning: certain pairs are excluded!";
/*
   if (UseBasketThreshold)
      {
      ls_Msg = ls_Msg + NL;
      if (gd_BasketBalance < BasketThreshold)
         {
         ls_Msg = ls_Msg + NL + "Basket threshold protection is not available!!!";
         ls_Msg = ls_Msg + NL + "Basket Balance less than " + BasketThreshold;
         }
      else
         ls_Msg = ls_Msg + NL + "Basket threshold protection will be applied!";
      }
*/         
   
   li_Ans = MessageBox(ls_Msg,"BUY Basket Confirmation",MB_YESNO|MB_ICONQUESTION);
   if (li_Ans==IDYES)
      Buy_Basket();
   else
      // Reset the button to original 
      Object_Create(ObjPfx+"Buy",BuyButX,BuyButY,"Buy Basket",ButtonSize,ButtonFont,BuyColor);
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| sell a basket (pre-processing)                                    |
//+------------------------------------------------------------------+
int Sell_Basket_PP()
  {
//----
   int li_Ans;
   string ls_Msg;
   
   ls_Msg = "Are you sure you wish to open a SELL basket?";
   
   if (gb_ExcludedPairs)
      ls_Msg = ls_Msg + NL + NL + "Warning: certain pairs are excluded!";
   
/*
   if (UseBasketThreshold)
      {
      ls_Msg = ls_Msg + NL;
      if (gd_BasketBalance > (BasketThreshold*(-1)))
         {
         ls_Msg = ls_Msg + NL + "Basket threshold protection is not available!!!";
         ls_Msg = ls_Msg + NL + "Basket Balance greater than " + (BasketThreshold*(-1));
         }
      else
         ls_Msg = ls_Msg + NL + "Basket threshold protection will be applied!";
      }
*/
   
   li_Ans = MessageBox(ls_Msg,"SELL Basket Confirmation",MB_YESNO|MB_ICONQUESTION);
   if (li_Ans==IDYES)
      Sell_Basket();
   else
      // Reset the button to original 
      Object_Create(ObjPfx+"Sell",SellButX,SellButY,"Sell Basket",ButtonSize,ButtonFont,SellColor);
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| pending buy basket (pre-processing)                              |
//+------------------------------------------------------------------+
int Pending_Buy_Basket_PP()
  {
//----
   int li_Ans;
   string ls_Msg;
   
   ls_Msg = "Are you sure you wish to open a Pending BUY?";

   if (gb_ExcludedPairs)
      ls_Msg = ls_Msg + NL + NL + "Warning: certain pairs are excluded!";
   
/*
   if (UseBasketThreshold)
      {
      ls_Msg = ls_Msg + NL;
      ls_Msg = ls_Msg + "Basket Threshold Protection is not available for pendings!";
      }
*/
   
   li_Ans = MessageBox(ls_Msg,"Pending BUY Confirmation",MB_YESNO|MB_ICONQUESTION);
   if (li_Ans==IDYES)
      Create_Pending_Line("BUY");
   else
      // Reset the button to original 
      Object_Create_2(ObjPfx+PenBuyText,PenBuyButX,PenBuyButY,PenBuyText,ButtonSize,ButtonFont,BuyColor);
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| pending sell basket (pre-processing)                             |
//+------------------------------------------------------------------+
int Pending_Sell_Basket_PP()
  {
//----
   int li_Ans;
   string ls_Msg;
   
   ls_Msg = "Are you sure you wish to open a Pending SELL?";
   
   if (gb_ExcludedPairs)
      ls_Msg = ls_Msg + NL + NL + "Warning: certain pairs are excluded!";
/*
   if (UseBasketThreshold)
      {
      ls_Msg = ls_Msg + NL;
      ls_Msg = ls_Msg + "Basket Threshold Protection is not available for pendings!";
      }
*/
   
   li_Ans = MessageBox(ls_Msg,"Pending SELL Confirmation",MB_YESNO|MB_ICONQUESTION);
   if (li_Ans==IDYES)
      Create_Pending_Line("SELL");
   else
      // Reset the button to original 
      Object_Create_2(ObjPfx+PenSellText,PenSellButX,PenSellButY,PenSellText,ButtonSize,ButtonFont,SellColor);
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| create a pending line                                            |
//+------------------------------------------------------------------+
int Create_Pending_Line(string ps_dir)
  {
//----
   string ls_name;
   color lc_color;
   double ld_price;
   
   if (ps_dir == "BUY")
      {
      ls_name = ObjPfx + PenBuyText + "_VL";
      lc_color = BuyColor;
      ld_price = Close[0] + DftPendingDistance;
      }

   if (ps_dir == "SELL")
      {
      ls_name = ObjPfx + PenSellText + "_VL";
      lc_color = SellColor;
      ld_price = Close[0] - DftPendingDistance;
      }
   
   ObjectCreate(ls_name,OBJ_HLINE,0,0,0);
   ObjectSet(ls_name,OBJPROP_PRICE1,ld_price);
   ObjectSet(ls_name,OBJPROP_WIDTH,1);
   ObjectSet(ls_name,OBJPROP_COLOR,lc_color);
   ObjectSet(ls_name,OBJPROP_STYLE,STYLE_SOLID);
   
   ObjectDelete(ObjPfx+PenBuyText);
   ObjectDelete(ObjPfx+PenSellText);
   gb_PendingExists = true;
   
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| close a basket                                                   |
//+------------------------------------------------------------------+
int Close_Basket()
  {
//----
   // Reset the minimum profit
//   GlobalVariableSet(Breakeven_GVar,0);
   GlobalVariableSet(Trailing_GVar,0);
   
   // Delete the button
   ObjectDelete(ObjPfx+"Close");
   
   Show_Processing(true);
   
   // Create the arrow 
   datetime ldt_Time = TimeCurrent();
   ObjectCreate(ArrowPfx+"Close Basket"+ldt_Time,OBJ_ARROW,0,0,0);
   ObjectSet(ArrowPfx+"Close Basket"+ldt_Time,OBJPROP_TIME1,Time[0]);
   ObjectSet(ArrowPfx+"Close Basket"+ldt_Time,OBJPROP_PRICE1,Close[0]);
   ObjectSet(ArrowPfx+"Close Basket"+ldt_Time,OBJPROP_ARROWCODE,3);
   ObjectSet(ArrowPfx+"Close Basket"+ldt_Time,OBJPROP_COLOR,CloseArrowColor);

   // Get open orders   
   int cnt;
   while(true)
      {
      for (int i=0; i<=OrdersTotal(); i++)
         {
         if (OrderSelect(i-1,SELECT_BY_POS)==true)
            {
            // Not this magic number - ignore
            if (OrderMagicNumber() != BasketMagicNbr) continue;
         
//         if (OrderType() == OP_BUY) Close_Buy(OrderTicket());
//         if (OrderType() == OP_SELL) Close_Sell(OrderTicket());
            Close_One_Trade(OrderTicket());
            }
         }
      if (!Open_Trades()) break;
      cnt++;
      if (cnt > MaxRetries)
         {
         Print("Something strange happened while closing the basket, check log");
         break;
         }
      Sleep(100);
      }
   
   gb_OpenBasket = false;
   
   Show_Processing(false);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| return true if open trades                                       |
//+------------------------------------------------------------------+
bool Open_Trades()
  {
//----
   for (int i=0; i<=OrdersTotal(); i++)
      {
      if (OrderSelect(i-1,SELECT_BY_POS)==true)
         {
         // Not this magic number - ignore
         if (OrderMagicNumber() != BasketMagicNbr) continue;
         return(true);
         }
      }
//----
   return(false);
  }
//+------------------------------------------------------------------+
//| buy a basket                                                     |
//+------------------------------------------------------------------+
int Buy_Basket()
  {
//----

   // Delete the button
   ObjectDelete(ObjPfx+"Buy");
   ObjectDelete(ObjPfx+PenBuyText);
   ObjectDelete(ObjPfx+PenBuyText+"_VL");
   
   Show_Processing(true);
   
   gb_BuyAllowed = false;
   
   // Reset the minimum profit
//   GlobalVariableSet(Breakeven_GVar,0);
   GlobalVariableSet(Trailing_GVar,0);
   

   // Create the arrow 
   datetime ldt_Time = TimeCurrent();
   ObjectCreate(ArrowPfx+"Buy Basket"+ldt_Time,OBJ_ARROW,0,0,0);
   ObjectSet(ArrowPfx+"Buy Basket"+ldt_Time,OBJPROP_TIME1,Time[0]);
   ObjectSet(ArrowPfx+"Buy Basket"+ldt_Time,OBJPROP_PRICE1,Close[0]);
   ObjectSet(ArrowPfx+"Buy Basket"+ldt_Time,OBJPROP_ARROWCODE,1);
   ObjectSet(ArrowPfx+"Buy Basket"+ldt_Time,OBJPROP_COLOR,BuyColor);
   
   
   double ld_price;
   int li_ticket;
   int li_type;
   
   for (int i=0; i<PairsInBasket; i++)
      {
      if (TradePair[i] == "") break;
      if (PairExcluded[i] != "") continue;
      if (TradePairOpp[i] == 1)
         li_type = OP_SELL;
      else
         li_type = OP_BUY;
      li_ticket = Open_One_Trade(TradePair[i],li_type,LotSize,EAComment,BasketMagicNbr);
      // For each pair get the ask 
/*      while(true)
         {
         ld_price = MarketInfo(TradePair[i],MODE_ASK);
         li_ticket = OrderSend(TradePair[i],OP_BUY,LotSize,ld_price,SlippagePts,0,0,EAComment,BasketMagicNbr,0,CLR_NONE);
         if (li_ticket > 0) break;
         }
*/
      }   

   gb_OpenBasket = true;
   
   Show_Processing(false);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| sell a basket                                                   |
//+------------------------------------------------------------------+
int Sell_Basket()
  {
//----
   // Delete the button
   ObjectDelete(ObjPfx+"Sell");
   ObjectDelete(ObjPfx+PenSellText);
   ObjectDelete(ObjPfx+PenSellText+"_VL");
   
   Show_Processing(true);
   
   gb_SellAllowed = false;

   // Reset the minimum profit
//   GlobalVariableSet(Breakeven_GVar,0);
   GlobalVariableSet(Trailing_GVar,0);

   // Create the arrow 
   datetime ldt_Time = TimeCurrent();
   ObjectCreate(ArrowPfx+"Sell Basket"+ldt_Time,OBJ_ARROW,0,0,0);
   ObjectSet(ArrowPfx+"Sell Basket"+ldt_Time,OBJPROP_TIME1,Time[0]);
   ObjectSet(ArrowPfx+"Sell Basket"+ldt_Time,OBJPROP_PRICE1,Close[0]);
   ObjectSet(ArrowPfx+"Sell Basket"+ldt_Time,OBJPROP_ARROWCODE,1);
   ObjectSet(ArrowPfx+"Sell Basket"+ldt_Time,OBJPROP_COLOR,SellColor);

   double ld_price;
   int li_ticket;
   int li_type;
   int li_err;
   for (int i=0; i<PairsInBasket; i++)
      {
      // For each pair get the bid 
      if (TradePair[i] == "") break;
      if (PairExcluded[i] != "") continue;
      if (TradePairOpp[i] == 1)
         li_type = OP_BUY;
      else
         li_type = OP_SELL;
      li_ticket = Open_One_Trade(TradePair[i],li_type,LotSize,EAComment,BasketMagicNbr);
/*
      while(true)
         {

         ld_price = MarketInfo(TradePair[i],MODE_BID);
         li_err = GetLastError();
         if (li_err != 0) {Print("Error Market Info ",li_err," Pair ",TradePair[i]); break;}
         li_ticket = OrderSend(TradePair[i],OP_SELL,LotSize,ld_price,SlippagePts,0,0,EAComment,BasketMagicNbr,0,CLR_NONE);
         if (li_err != 0) {Print("Error ",li_err); break;}
         if (li_ticket > 0) break;
         }
*/
      }   

   gb_OpenBasket = true;
   
   Show_Processing(false);
//----
   return(0);
  }
/*
//+------------------------------------------------------------------+
//| close buy                                                   |
//+------------------------------------------------------------------+
int Close_Buy(int pi_Ticket)
  {
//----
   bool lb_ans = OrderSelect(pi_Ticket,SELECT_BY_TICKET);
   if (!lb_ans) return(0);
   
   double ld_price = MarketInfo(OrderSymbol(),MODE_BID);
   
   OrderClose(pi_Ticket,OrderLots(),ld_price,SlippagePts,CLR_NONE);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| close sell                                                   |
//+------------------------------------------------------------------+
int Close_Sell(int pi_Ticket)
  {
//----
   bool lb_ans = OrderSelect(pi_Ticket,SELECT_BY_TICKET);
   if (!lb_ans) return(0);
   
   double ld_price = MarketInfo(OrderSymbol(),MODE_ASK);
   
   OrderClose(pi_Ticket,OrderLots(),ld_price,SlippagePts,CLR_NONE);
//----
   return(0);
  }
*/
//+------------------------------------------------------------------+
//| open one trade                                                   |
//+------------------------------------------------------------------+
int Open_One_Trade(string ps_Symbol, int pi_Type, double pd_LotSize,
                   string ps_Comment, int pi_MagicNbr)
  {
//----
   int pi_Ticket;
   double ld_Price;
   
   int li_Cnt;
   
   while(true)
      {
      
      double ld_Bid = MarketInfo(ps_Symbol,MODE_BID);
      double ld_Ask = MarketInfo(ps_Symbol,MODE_ASK);
      
      if (pi_Type == OP_BUY)
         ld_Price = ld_Ask;
      if (pi_Type == OP_SELL)
         ld_Price = ld_Bid;
         
      // Send the trade
      
      pi_Ticket = OrderSend(ps_Symbol,pi_Type,pd_LotSize,ld_Price,SlippagePts,0,0,ps_Comment,pi_MagicNbr,0,CLR_NONE);
      
      if (pi_Ticket > 0) break;
      
      // Otherwise we had an error, so sleep a while and try again
      li_Cnt++;
      if (li_Cnt > MaxRetries)
         {
         Print("We had a major error sending trade for ",ps_Symbol," please check logs");
         break;
         }
      Sleep(100);
      }
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| close one trade                                                  |
//+------------------------------------------------------------------+
int Close_One_Trade(int pi_Ticket)
  {
//----
   int li_Cnt;
   bool lb_ans;
   
   while(true)
      {
      lb_ans = OrderSelect(pi_Ticket,SELECT_BY_TICKET);
      if (!lb_ans) break;
      lb_ans = OrderClose(pi_Ticket,OrderLots(),OrderClosePrice(),SlippagePts,CLR_NONE);
      if (lb_ans) break;
      li_Cnt++;
      if (li_Cnt > MaxRetries)
         {
         Print("We had a major error closing ticket ",pi_Ticket," check the logs");
         return(0);
         }
      Sleep(100);
      }
      
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| check to see if we need to open pending baskets                  |
//+------------------------------------------------------------------+
bool Open_Pendings()
  {
//----   
   double ld_price;
   
   ld_price = ObjectGet(ObjPfx+PenBuyText+"_VL",OBJPROP_PRICE1);
   if (ld_price != 0)
      {
      if (Close[0] > ld_price)
         Buy_Basket();
         return(true);
      }
      
   ld_price = ObjectGet(ObjPfx+PenSellText+"_VL",OBJPROP_PRICE1);
   if (ld_price != 0)
      {
      if (Close[0] < ld_price)
         Sell_Basket();
         return(true);
      }
//----
   return(false);
  }
//+------------------------------------------------------------------+
//| check trade conditions for new entries                           |
//+------------------------------------------------------------------+
int Check_Trade_Conditions()
  {
//----
   bool lb_MidLong,lb_MidShort;
   bool lb_UppShort;
   bool lb_LowLong;
   
   if (Direction == 0 || Direction == 6) return(0);
   
   if (Time[0] == LastAlert) return(0);
   
   // Set up the allowable bands based on the direction
   switch (Direction)
      {
      // Long
      case 1:  lb_MidLong = true;
               lb_LowLong = true;
               break;
      // Short 
      case 2:  lb_MidShort = true;
               lb_UppShort = true;
               break;
      // Range 
      case 3:  lb_LowLong = true;
               lb_UppShort = true;
               break;
      // CT Long 
      case 4:  lb_LowLong = true;
               lb_MidLong = true;
               lb_UppShort = true;
               break;
      // CT Short 
      case 5:  lb_UppShort = true;
               lb_MidShort = true;
               lb_LowLong = true;
               break;
      }
      
   // Get the bands
   double ld_Upper = iBands(NULL,0,20,2,0,PRICE_CLOSE,MODE_UPPER,1);
   double ld_Lower = iBands(NULL,0,20,2,0,PRICE_CLOSE,MODE_LOWER,1);
   double ld_Mid = iBands(NULL,0,20,2,0,PRICE_CLOSE,MODE_MAIN,1);
   
   // Send any alerts necessary
   if (lb_UppShort &&
       Open[1] > ld_Upper && Close[1] < ld_Upper)
      {
      Alert("Possible SHORT opportunity at ",TimeToStr(Time[0],TIME_DATE|TIME_MINUTES));
      LastAlert = Time[0];
      }
   
   if (lb_MidShort &&
       Open[1] > ld_Mid && Close[1] < ld_Mid)
      {
      Alert("Possible SHORT opportunity at ",TimeToStr(Time[0],TIME_DATE|TIME_MINUTES));
      LastAlert = Time[0];
      }
          
   if (lb_LowLong &&
       Open[1] < ld_Lower && Close[1] > ld_Lower)
      {
      Alert("Possible LONG opportunity at ",TimeToStr(Time[0],TIME_DATE|TIME_MINUTES));
      LastAlert = Time[0];
      }
   
   if (lb_MidLong &&
       Open[1] < ld_Mid && Close[1] > ld_Mid)
      {
      Alert("Possible LONG opportunity at ",TimeToStr(Time[0],TIME_DATE|TIME_MINUTES));
      LastAlert = Time[0];
      }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| display the user comments                                        |
//+------------------------------------------------------------------+
int Display_Comments()
  {
//----
   string ls_Comment;
   int li_value;
   double ld_value;
   
   // Pairs in the basket and suffix
   ls_Comment = NL + "No. of Basket Pairs: " + PairsInBasket + " (Pair Suffix: "
                + PairSuffix + ")";
   ls_Comment = ls_Comment + NL + "Pairs to be traded oppositely: " + PairsToTradeOpposite;
   
   ls_Comment = ls_Comment + NL + "Lot Size: " + DoubleToStr(LotSize,2);
                
   // NanningBob and Trade condition settings
   ls_Comment = ls_Comment + NL + NL + "Use NanningBob 10.0: " + BoolToStr(UseNanningBobBasis);
   ls_Comment = ls_Comment + NL + "Use EA Trade Conditions: " + BoolToStr(UseTradeConditions);
             
   ls_Comment = ls_Comment + NL + NL + "Use Take Profit: " + BoolToStr(UseTakeProfit);

   if (UseTakeProfit)
      {
      if (VisualTP)
         {
//         li_value = ObjectGet(ObjPfx+"TakeProfit",OBJPROP_PRICE1);
//         if (li_value != 0)
//            li_value = MathAbs(Close[0] - li_value);
//         else
//         li_value = DefaultTP;
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Visual Take Profit line shown";
         }
      if (ValueTP)
         {
         ld_value = GlobalVariableGet(TakeProfit_GVar);
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Value Take Profit at " + DoubleToStr(ld_value,2) + " " + AccountCurrency();
         }
      }

   ls_Comment = ls_Comment + NL + NL + "Use Stop Loss: " + BoolToStr(UseStopLoss);
   if (UseStopLoss)
      {
      if (VisualSL)
         {
//         li_value = ObjectGet(ObjPfx+"StopLoss",OBJPROP_PRICE1);
//         if (li_value != 0)
//            li_value = MathAbs(Close[0] - li_value);
//         else
//         li_value = DefaultSL;
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Visual Stop Loss line shown";
         }
      if (ValueSL)
         {
         ld_value = GlobalVariableGet(StopLoss_GVar);
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Value Stop Loss at " + DoubleToStr(ld_value,2) + " " + AccountCurrency();
         }
      }

   ls_Comment = ls_Comment + NL + NL + "Use Trailing: " + BoolToStr(UseTrailing);
   
   if (UseTrailing)
      {
      ld_value = GlobalVariableGet(DftBrkEv_GVar);
      ls_Comment = ls_Comment + NL + ".........";
      ls_Comment = ls_Comment + " Set to B/E + Lock at " + DoubleToStr(ld_value,2) + " " + AccountCurrency();
      ld_value = GlobalVariableGet(DftTrail_GVar);
      ls_Comment = ls_Comment + NL + ".........";
      ls_Comment = ls_Comment + " Start trailing at " + DoubleToStr(ld_value,2) + " " + AccountCurrency();
      
      if (TrailFixed)
         {
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Fixed trailing at " + DoubleToStr(FixedTrailValue,2) + " " + AccountCurrency();
         }
      if (TrailPercent)      
         {
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Percentage trailing at " + DoubleToStr(TrailPrcntage,0) + "%";
         }
         
      }

   // Show the current trailing / minimum profit value
   double ld_Value = GlobalVariableGet(Trailing_GVar);
   ls_Comment = ls_Comment + NL + NL + "Current Minimum Profit is " + DoubleToStr(ld_Value,2) + " " + AccountCurrency();


   ls_Comment = ls_Comment + NL + NL + "Hedging Allowed: " + BoolToStr(AllowHedging);
   
   if (AllowHedging)
      {
      ls_Comment = ls_Comment + NL + ".........";
      ls_Comment = ls_Comment + " Minimum Loss per pair to hedge: " + DoubleToStr(PairMinLoss4Hedge,2) + " " + AccountCurrency();
      if (VisualHedge)
         {
//         li_value = ObjectGet(ObjPfx+"HedgeLine",OBJPROP_PRICE1);
//         if (li_value != 0)
//            li_value = MathAbs(Close[0] - li_value);
//         else
//         li_value = DefaultHedge;
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Visual Hedge line shown";
         }
      if (ValueHedge)
         {
         ld_value = GlobalVariableGet(Hedge_GVar);
         ls_Comment = ls_Comment + NL + ".........";
         ls_Comment = ls_Comment + " Value Hedge at " + DoubleToStr(ld_value,2) + " " + AccountCurrency();
         }
      }

   // Show the default & actual breakeven value
//   if (!UseTrailing)
//      {
//      ls_Comment = ls_Comment + NL + NL + "Default Breakeven at " + DoubleToStr(DefaultBreakevenValue,2) + " " + AccountCurrency();
//      ld_Value = GlobalVariableGet(Breakeven_GVar);
//      ls_Comment = ls_Comment + NL + "Actual Breakeven at " + DoubleToStr(ld_Value,2) + " " + AccountCurrency();
//      }

/*
   // Show the basket threshold & current balance
   ls_Comment = ls_Comment + NL + NL + "Use Basket Threshold Protection: " + BoolToStr(UseBasketThreshold);

   if (UseBasketThreshold)
      {
      ls_Comment = ls_Comment + NL + ".........";
      ls_Comment = ls_Comment + " Basket Threshold (+/-): " + BasketThreshold;
      ls_Comment = ls_Comment + NL + ".........";
      ls_Comment = ls_Comment + " Pair Threshold (+/-): " + PairThreshold;
      }
*/
   // Show if there is excluded pairs
   ls_Comment = ls_Comment + NL + NL + "Excluded pairs: " + BoolToStr(gb_ExcludedPairs);
         
   ls_Comment = ls_Comment + NL + NL + "Basket Balance: " + DoubleToStr(gd_BasketBalance,0);
   ls_Comment = ls_Comment + " / " + DoubleToStr(gd_BasketBalPc,2) + " %";
      
   Comment(ls_Comment);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Show Pair Rankings                                               |
//+------------------------------------------------------------------+
int Show_Pair_Rankings(bool pb_Basket)
  {
//----
   int i;
   int idx;
   int i2;
   string ls_pair;
   color lc_color;
   
   // If there is no basket show the general balance
   if (!pb_Basket)
      {
      for (i=0; i<PairsInBasket; i++)
         {
//         ObjectDelete(ObjPfx+"Rank"+i);
         ObjectDelete(ObjPfx+"MisPair"+i);
         }
//      return(0);
      }
     
   Show_Basket_Balance(pb_Basket);

   // Otherwise show the rankings      
   if (!pb_Basket) return(0);
   
   // First of all get rid of any pairs that no longer have an order 
   ArrayCopy(PairRank3,PairRank,0,0,WHOLE_ARRAY);
   ArraySort(PairRank,0,0,MODE_DESCEND);
   ArrayInitialize(PairRank2,(-1));
   i2 = 0;
   for (i=0; i<PairsInBasket; i++)
      if (PairRank[i][1] != -1)
         {
         PairRank2[i2][0] = PairRank[i][0];
         PairRank2[i2][1] = PairRank[i][1];
         i2++;
         }
   
   int OffsetY = RankStartY;
//   ArraySort(PairRank,0,0,MODE_DESCEND);
   for (i=0; i<PairsInBasket; i++)
      {
      idx = PairRank2[i][1];
      
      string ls_string = TradePair[idx] + " " + DoubleToStr(PairRank2[i][0],2) +
                         " (" + DoubleToStr(PairStrength[idx],0) + ")";
      
      // Set the color 
      lc_color = DftTextColor;
      if (PairRank2[i][0] < 0)
         lc_color = LossColor;
      if (PairRank2[i][0] > 0)
         lc_color = ProfitColor;
      if (PairRank2[i][1] != (-1))
         Object_Create(ObjPfx+"Rank"+i,RankX[i],OffsetY,ls_string,RankFontSize,DftFont,lc_color);
      else
         ObjectDelete(ObjPfx+"Rank"+i);
      OffsetY += RankOffY[i];
      }
      
   // Now show the missing pairs (and those that are hedged later)
//   for (i=0; i<MaxPairs; i++)
//      if (PairRank3[i][1] != (-1))
//         ObjectDelete(ObjPfx+"MisPair"+i);
      
   OffsetY = MisPairStartY;
   i2 = -1;
   for (i=0; i<PairsInBasket; i++)
      {
      if (PairRank3[i][1] == (-1))
         {
         i2++;
         ls_string = TradePair[i] + " (" + DoubleToStr(PairStrength[i],0) + ")";
         
         Object_Create(ObjPfx+"MisPair"+i2,RankX[i2],OffsetY,ls_string,RankFontSize,DftFont,DftTextColor);
         OffsetY += RankOffY[i2];
         }
      }
         
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| show the general basket balance                                  |
//+------------------------------------------------------------------+
int Show_Basket_Balance(bool pb_DontDisplay)
  {
//----
   double ld_MA.Now, ld_MA.Prv;
   double ld_Balance;
   double ld_TotalBalance;
   double ld_Point;
   double ld_Buy, ld_Sell;
   int li_Exclusion;
   string ls_string;
   color lc_color;
   
//   Print("Display basket ",pb_DontDisplay);
   ArrayInitialize(PairStrength,0);
   
   int OffsetY = RankStartY;
   for (int i=0; i<PairsInBasket; i++)
      {
      // Get the MA's for now and the lookback
      if (TradePair[i] != "")
         {
         ld_Point = MarketInfo(TradePair[i],MODE_POINT);
         if (ld_Point == 0) ld_Point = 1;
         ld_MA.Now = iMA(TradePair[i],0,BasketBalanceMA,0,MODE_EMA,PRICE_CLOSE,0);
         ld_MA.Prv = iMA(TradePair[i],0,BasketBalanceMA,0,MODE_EMA,PRICE_CLOSE,BasketBalanceLB);
         
         ld_Balance = (ld_MA.Now - ld_MA.Prv) / ld_Point;
         if (TradePairOpp[i] == 1) ld_Balance *= (-1);
         
         PairStrength[i] = ld_Balance;
         ld_TotalBalance += ld_Balance;
                  
         // Set the color 
         lc_color = DftTextColor;
         if (ld_Balance < 0)
            {
            lc_color = SellColor;
            ld_Sell++;
            }
         if (ld_Balance > 0)
            {
            lc_color = BuyColor;
            ld_Buy++;
            }
         
         ls_string = PairExcluded[i] + TradePair[i] + " (" + DoubleToStr(ld_Balance,0) + ")";
         if (!pb_DontDisplay)
            Object_Create(ObjPfx+"Rank"+i,RankX[i],OffsetY,ls_string,RankFontSize,DftFont,lc_color);
         OffsetY += RankOffY[i];
         }
      }

   // Now create the total balance display
   lc_color = DftTextColor;
   gd_BasketBalPc = 0;
   if (ld_TotalBalance < 0)
      {
      lc_color = SellColor;
      gd_BasketBalPc = ld_Sell / PairsInBasket * 100;
      }
   if (ld_TotalBalance > 0)
      {
      lc_color = BuyColor;
      gd_BasketBalPc = ld_Buy / PairsInBasket * 100;
      }
      
   
   ls_string = "Basket Balance " + DoubleToStr(ld_TotalBalance,0) +
                      " / " + DoubleToStr(gd_BasketBalPc,2) + " %";
   if (!pb_DontDisplay)
      Object_Create(ObjPfx+"TotalBalance",TotBalanceX,TotBalanceY,ls_string,RankFontSize,DftFont,lc_color);
      
   if (pb_DontDisplay)
      ObjectDelete(ObjPfx+"TotalBalance");
      
   gd_BasketBalance = ld_TotalBalance;         
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| recreate any missing global variables                            |
//+------------------------------------------------------------------+
int Recreate_Global_Variables()
  {
//----
//   if (!GlobalVariableCheck(Breakeven_GVar))
//      GlobalVariableSet(Breakeven_GVar,0);
   if (!GlobalVariableCheck(Trailing_GVar))
      GlobalVariableSet(Trailing_GVar,0);
   if (!GlobalVariableCheck(TakeProfit_GVar))
      GlobalVariableSet(TakeProfit_GVar,DefaultTPValue);
   if (!GlobalVariableCheck(StopLoss_GVar))
      GlobalVariableSet(StopLoss_GVar,DefaultSLValue);
   if (!GlobalVariableCheck(Hedge_GVar))
      GlobalVariableSet(Hedge_GVar,DefaultHedgeValue);
         
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| reset pair exclusion                                             |
//+------------------------------------------------------------------+
int Reset_Pair_Exclusion()
  {
//----
   for (int i=0; i<PairsInBasket; i++)
      if (TradePair[i] != "")
         GlobalVariableSet(Exclusion_GVar+TradePair[i],0);
//   if (!GlobalVariableCheck(Breakeven_GVar))
//      GlobalVariableSet(Breakeven_GVar,0);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| get pair exclusions                                              |
//+------------------------------------------------------------------+
int Get_Pair_Exclusions()
  {
//----
   int li_excl;
   gb_ExcludedPairs = false;
   
   for (int i=0; i<PairsInBasket; i++)
      {
      if (TradePair[i] != "")
         {
         li_excl = GlobalVariableGet(Exclusion_GVar+TradePair[i]);
         if (li_excl > 0)
            {
            PairExcluded[i] = "**";
            gb_ExcludedPairs = true;
            }
         else
            PairExcluded[i] = "";
         }
      }
         
//   if (!GlobalVariableCheck(Breakeven_GVar))
//      GlobalVariableSet(Breakeven_GVar,0);
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Show/Delete Processing text                                      |
//+------------------------------------------------------------------+
int Show_Processing(bool pi_show)
  {
//----
   if (pi_show)
      Object_Create(ObjPfx+"Processing",ProcessingX,ProcessingY,"PROCESSING",ProcessingSize,ProcessingFont,ProcessingColor);
   else
      ObjectDelete(ObjPfx+"Processing");
   
   WindowRedraw();   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Boolean to string                                                |
//+------------------------------------------------------------------+
string BoolToStr(bool pi_bool)
  {
//----
   string ps_str;
   if (pi_bool) ps_str = "Yes";
   if (!pi_bool) ps_str = "No";
//----
   return(ps_str);
  }
//+------------------------------------------------------------------+
//| get the applicable trade pairs for the basket number             |
//+------------------------------------------------------------------+
int Get_Trade_Pairs()
  {
//----
   
   for (int i=0; i<PairsInBasket; i++)
      {
      TradePair[i] = "";
      OppTradePairs[i] = "";
      TradePairOpp[i] = 0;
      }
      
   switch (PairsInBasket)
      {
      case 1:  TradePair[0] = "GBPJPY" + PairSuffix;
               break;
      case 2:  TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURUSD" + PairSuffix;
               break;
      case 4:  TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURJPY" + PairSuffix;
               TradePair[2] = "EURUSD" + PairSuffix;
               TradePair[3] = "GBPJPY" + PairSuffix;
               break;
      case 6:  TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURJPY" + PairSuffix;
               TradePair[2] = "AUDUSD" + PairSuffix;
               TradePair[3] = "EURUSD" + PairSuffix;
               TradePair[4] = "GBPJPY" + PairSuffix;
               TradePair[5] = "NZDUSD" + PairSuffix;
               break;
      case 8:  TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURJPY" + PairSuffix;
               TradePair[2] = "AUDUSD" + PairSuffix;
               TradePair[3] = "NZDJPY" + PairSuffix;
               TradePair[4] = "EURUSD" + PairSuffix;
               TradePair[5] = "GBPJPY" + PairSuffix;
               TradePair[6] = "NZDUSD" + PairSuffix;
               TradePair[7] = "AUDJPY" + PairSuffix;
               break;
      case 10: TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURGBP" + PairSuffix;
               TradePair[2] = "GBPJPY" + PairSuffix;
               TradePair[3] = "CADJPY" + PairSuffix;
               TradePair[4] = "NZDUSD" + PairSuffix;
               TradePair[5] = "EURUSD" + PairSuffix;
               TradePair[6] = "USDJPY" + PairSuffix;
               TradePair[7] = "AUDUSD" + PairSuffix;
               TradePair[8] = "NZDJPY" + PairSuffix;
               TradePair[9] = "GBPCHF" + PairSuffix;
               break;
      case 12: TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURGBP" + PairSuffix;
               TradePair[2] = "GBPJPY" + PairSuffix;
               TradePair[3] = "CADJPY" + PairSuffix;
               TradePair[4] = "NZDUSD" + PairSuffix;
               TradePair[5] = "AUDJPY" + PairSuffix;
               TradePair[6] = "EURUSD" + PairSuffix;
               TradePair[7] = "USDJPY" + PairSuffix;
               TradePair[8] = "AUDUSD" + PairSuffix;
               TradePair[9] = "NZDJPY" + PairSuffix;
               TradePair[10] = "GBPCHF" + PairSuffix;
               TradePair[11] = "CHFJPY" + PairSuffix;
               break;
      case 14: TradePair[0] = "GBPUSD" + PairSuffix;
               TradePair[1] = "EURGBP" + PairSuffix;
               TradePair[2] = "GBPJPY" + PairSuffix;
               TradePair[3] = "USDCHF" + PairSuffix;
               TradePair[4] = "NZDUSD" + PairSuffix;
               TradePair[5] = "AUDJPY" + PairSuffix;
               TradePair[6] = "EURJPY" + PairSuffix;
               TradePair[7] = "EURUSD" + PairSuffix;
               TradePair[8] = "USDJPY" + PairSuffix;
               TradePair[9] = "AUDUSD" + PairSuffix;
               TradePair[10] = "NZDJPY" + PairSuffix;
               TradePair[11] = "GBPCHF" + PairSuffix;
               TradePair[12] = "CHFJPY" + PairSuffix;
               TradePair[13] = "EURCHF" + PairSuffix;
               break;
       default: break;
       }
       
   // We now need to extract the pairs to trade opposite to the basket direction
	StrPairToStringArray(PairsToTradeOpposite, OppTradePairs, PairSuffix);
	
	// Now go through and update the array used in calculations
	for (i=0; i<PairsInBasket; i++)
	   if (Get_Opposite_Pair_Index(TradePair[i]) != (-1))
	      TradePairOpp[i] = 1;

       
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| get a currency pair index                                        |
//+------------------------------------------------------------------+
int Get_Pair_Index(string ps_Pair)
  {
//----
   for (int i=0; i < PairsInBasket; i++)
      if (ps_Pair == TradePair[i]) return(i);
//----
   return(-1);
  }  
//+------------------------------------------------------------------+
//| get a opposite currency pair index                               |
//+------------------------------------------------------------------+
int Get_Opposite_Pair_Index(string ps_Pair)
  {
//----
   for (int i=0; i < PairsInBasket; i++)
      if (ps_Pair == OppTradePairs[i]) return(i);
//----
   return(-1);
  }  
//+------------------------------------------------------------------+
//| convert string to an array                                       |
//+------------------------------------------------------------------+
void StrPairToStringArray(string str, string &a[], string p_suffix, string delim=",")
  {
//----
	int z1=-1, z2=0;
	for (int i=0; i<ArraySize(a); i++)
   	{
		z2 = StringFind(str,delim,z1+1);
		a[i] = StringSubstr(str,z1+1,z2-z1-1) + p_suffix;
		if (z2 >= StringLen(str)-1)   break;
		z1 = z2;
	   }
//----
	return(0);
  }
/*
//+------------------------------------------------------------------+
//| count a string in a string                                       |
//+------------------------------------------------------------------+
int StringFindCount(string str, string str2)
  {
//----
   int c = 0;
   for (int i=0; i<StringLen(str); i++)
     if (StringSubstr(str,i,StringLen(str2)) == str2)  c++;
//----
   return(c);
  }
*/
//+------------------------------------------------------------------+
//| create screen objects                                            |
//+------------------------------------------------------------------+
void Object_Create(string ps_name,int pi_x,int pi_y,string ps_text=" ",int pi_size=12,
                  string ps_font="Arial",color pc_colour=CLR_NONE)
  {
//----
   
   ObjectCreate(ps_name,OBJ_LABEL,0,0,0,0,0);
   ObjectSet(ps_name,OBJPROP_CORNER,1);
   ObjectSet(ps_name,OBJPROP_COLOR,pc_colour);
   ObjectSet(ps_name,OBJPROP_XDISTANCE,pi_x);
   ObjectSet(ps_name,OBJPROP_YDISTANCE,pi_y);
   
   ObjectSetText(ps_name,ps_text,pi_size,ps_font,pc_colour);

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| create screen objects (bottom right)                             |
//+------------------------------------------------------------------+
void Object_Create_2(string ps_name,int pi_x,int pi_y,string ps_text=" ",int pi_size=12,
                  string ps_font="Arial",color pc_colour=CLR_NONE)
  {
//----
   
   ObjectCreate(ps_name,OBJ_LABEL,0,0,0,0,0);
   ObjectSet(ps_name,OBJPROP_CORNER,3);
   ObjectSet(ps_name,OBJPROP_COLOR,pc_colour);
   ObjectSet(ps_name,OBJPROP_XDISTANCE,pi_x);
   ObjectSet(ps_name,OBJPROP_YDISTANCE,pi_y);
   
   ObjectSetText(ps_name,ps_text,pi_size,ps_font,pc_colour);

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Delete the consolidation zones                                   |
//+------------------------------------------------------------------+
void Delete_Objects() 
  {
//----
   string ls_Obj.Name;
   for (int i=ObjectsTotal()-1; i>=0; i--)
      {
      ls_Obj.Name=ObjectName(i);
      if (StringFind(ls_Obj.Name,ObjPfx)>-1) ObjectDelete(ls_Obj.Name);
      }
   ObjectsRedraw();
//----
   return(0);
  }
//+------------------------------------------------------------------+